// /**
//  * @file
//  * Vue app for cart.
//  */
(function ($) {
  "use strict";

  if (document.readyState != 'loading') {
    vueCartApp("#app-cart");
  } else {
    window.addEventListener("DOMContentLoaded", function () {
      vueCartApp("#app-cart");
    });
  }
  
  function vueCartApp(sel) {
    var app;
    const elements = document.querySelectorAll(sel);
    const each = Array.prototype.forEach;
    each.call(elements, function (el, i) {
      let comp_template = el.querySelector('cart');
      app = Vue.createApp({ delimiters, comments, el }).use(store).use($cookies);
      app.component("favorite-field", {
        template: `<div
        class="cart-button cart-button--fav"
        :class="fav ? 'cart-button--fav-in' : ''"
        style="display:none"
        v-show="productsLoaded"
        @click="toggleFav"
        >
          <i class="far fa-heart"></i>
        </div>`,
        delimiters: delimiters,
        props: {
          pid: 0,
        },
        data() {
          return {
            productsLoaded: false,
            fav: false,
          };
        },
        methods: {
          toggleFav() {
            if (this.fav) {
              this.deleteFav();
            } else {
              this.addFav();
            }
          },
          addFav() {
            this.fav = true;
            this.likeClicked = true;
            var favorites = this.$cookies.get("favorites");
            if (typeof favorites !== "object" || favorites == null) {
              favorites = {};
            }
            favorites[this.pid] = true;
            this.$cookies.set("favorites", favorites, "30d");
            var total = Object.keys(favorites).length;
            this.$store.commit("updateFavorite", { total });
          },
          // Удалить из избранное.
          deleteFav() {
            this.fav = false;
            this.likeClicked = false;
            var favorites = this.$cookies.get("favorites");
            if (typeof favorites !== "object" || favorites == null) {
              favorites = {};
            }
            delete favorites[this.pid];
            this.$cookies.set("favorites", favorites, "30d");
            if (window.location.pathname == "/favorites") {
              window.location.reload();
            } else {
              var total = Object.keys(favorites).length;
              this.$store.commit("updateFavorite", { total });
            }
          },
        },
        created() {
          this.productsLoaded = true;
          let favorites = this.$cookies.get("favorites");
          if (
            favorites !== null &&
            typeof favorites == "object" &&
            favorites.hasOwnProperty(this.pid)
          ) {
            this.fav = true;
          }
          if (localStorage.watched) {
            var array = localStorage.watched.split("|");
          } else {
            var array = [];
          }
          let index = array.indexOf(this.pid.toString());
          if (index === -1) {
            array.push(this.pid);
            localStorage.watched = array.join("|");
          }
        },
      });
      app.component("cart", {
        template: comp_template,
        delimiters: delimiters,
        data() {
          return {
            ready: false,
            init: false,
            computed: false,
            quantity: {},
          };
        },
        computed: {
          cart() {
            return this.$store.state.cart;
          },
        },
        watch: {
          cart(newCart, oldCart) {
            if (!this.init) {
              this.init = true;
              return;
            }
            if (!this.computed) {
              this.computed = true;
              return;
            }
            let oldCartItems = {};
            let newCartItems = {};
            let items_old = Object.values(oldCart.items);
            items_old.forEach(element => {
              oldCartItems[element.vid] = element;
            });
            let items_new = Object.values(newCart.items);
            items_new.forEach(element => {
              newCartItems[element.vid] = element;
            });
            let newCartItemsLength = Object.keys(newCartItems).length;
            let oldCartItemsLength = Object.keys(oldCartItems).length;
            if (newCartItemsLength == oldCartItemsLength) {
              for (const [key, element] of Object.entries(newCartItems)) {
                let oldQuantity = +oldCartItems[key].quantity;
                if (oldQuantity != element.quantity) {
                  let changeQuantity = element.quantity - oldQuantity;
                  if (changeQuantity > 0) {
                    dataLayerEvent('add', element, Math.abs(changeQuantity));
                  }
                  if (changeQuantity < 0) {
                    dataLayerEvent('remove', element, Math.abs(changeQuantity));
                  }
                }
              }
            } else if (newCartItemsLength > oldCartItemsLength) {
              for (const [key, element] of Object.entries(newCartItems)) {
                if (oldCartItems[key] == undefined) {
                  dataLayerEvent('add', element, element.quantity);
                }
              }
            } else if (newCartItemsLength < oldCartItemsLength) {
              for (const [key, element] of Object.entries(oldCartItems)) {
                if (newCartItems[key] == undefined) {
                  dataLayerEvent('remove', element, element.quantity);
                }
              }
            }
          }
        },
        methods: {
          itemSumAdjustmentAmount(item) {
            let amount = 0;
            let symbol = '';
            let adjustment = item.adjustments;
            if (adjustment === undefined) {
              amount = item.totalPrice;
            }
            else {
              let total = item.number * item.quantity;
              adjustment.forEach((v) => {
                amount += +v.number;
                symbol = ' ' + v.currencySymbol;
              });
              amount += total;
              amount += ' ' + symbol;
            }
            return amount;
          },
          itemTotalPrice(item) {
            if (item.adjustments !== undefined) {
              let total = item.number * item.quantity;
              total += ' ' + item.adjustments[0].currencySymbol;
              return total;
            } 
            else {
              return;
            }
          },
          itemTotalDiscount(item) {
            let amount = 0;
            let adjustment = item.adjustments;
            if (adjustment !== undefined) {
              adjustment.forEach((v) => {
                amount += +v.number;
              });
              amount = Math.abs(amount) + ' ' + adjustment[0].currencySymbol;
              return amount;
            }
            else {
              return;
            }
          },
          productName(item) {
            let t = item.title;
            if (item.uname != '') {
              t += ', ' + item.uname;
            }
            return t;
          },
          initCart() {
            let loadPath = "/api/cart-load";
            let path = drupalSettings.path.currentPath;
            let arg = path.split("/");
            if (arg.length == 3) {
              if (arg[0] == "checkout" && parseInt(arg[1]) > 0) {
                loadPath += "/" + arg[1];
              }
            }
            axios.get(loadPath).then((response) => {
              this.$store.commit("updateCart", response.data);
              if (response.data.quantity > 0) {
                this.ready = true;
                this.fillQuantity();
              }
            });
          },
          fillQuantity() {
            this.quantity = {};
            this.cart.items.forEach(element => {
              this.quantity[element.vid] = +element.quantity;
            });
          },
          NumbersOnly(event) {
            event = (event) ? event : window.event;
            var charCode = (event.which) ? event.which : event.keyCode;
            if ((charCode > 31 && (charCode < 48 || charCode > 57)) && charCode !== 46) {
              event.preventDefault();
            } else {
              return true;
            }
          },
          editNote(item) {
            axios
              .post("/cart/set-item-note", {
                itemId: item.id,
                note: item.data_note,
              }).then(
                (response) => {
                  this.$store.commit("updateCart", response.data);
                },
              );
          },
          readQuantity(item) {
            let item_quantity = +this.quantity[item.vid];
            if (item.checkstore) {
              let steps = parseInt(item_quantity / item.qstep);
              if (steps < 1) {
                steps = 1;
                item_quantity = item.qstep;
              }
              if (item_quantity % item.qstep > 0) {
                item_quantity = steps * item.qstep;
              }
              if (item.quantity > item.stock) {
                let quantity = 0;
                while ((quantity + item.qstep) <= item.stock) {
                  quantity += item.qstep;
                }
                item_quantity = quantity;
              }
            }
            axios.post("/cart/set-product-quantity", {
              vid: item.id,
              quantity: item_quantity,
            }).then(
              (response) => {
                this.$store.commit("updateCart", response.data);
                this.fillQuantity();
              },
            );
          },
          changeQuantity(sign, item) {
            let item_quantity = +this.quantity[item.vid];
            switch (sign) {
              case 'plus':
                item_quantity += item.qstep;
                if (item.checkstore) {
                  if (item_quantity > item.stock) {
                    item_quantity -= item.qstep;
                  }
                }
                break;
              case 'minus':
                item_quantity -= item.qstep;
                if (item_quantity <= 0) {
                  item_quantity += item.qstep;
                }
                break;
            }
            axios
              .post("/cart/set-product-quantity", {
                vid: item.id,
                quantity: item_quantity,
              }).then(
                (response) => {
                  this.$store.commit("updateCart", response.data);
                  this.fillQuantity();
                },
              );
          },
          removeItem(vid) {
            axios.post("/cart/remove-cart-item", { vid: vid }).then((response) => {
              this.ready = false;
              this.initCart();
            });
          },
          numberFormat(number, step, decimals = 2, dec_point = ".", thousands_sep = " ") {
            let s_number =
              Math.abs(parseInt((number = (+number || 0).toFixed(decimals)))) + "";
            let len = s_number.length;
            let tchunk = len > 3 ? len % 3 : 0;
            let ch_first = tchunk ? s_number.substr(0, tchunk) + thousands_sep : "";
            let ch_rest = s_number
              .substr(tchunk)
              .replace(/(\d\d\d)(?=\d)/g, "$1" + thousands_sep);
            let ch_last = decimals
              ? dec_point + (Math.abs(number) - s_number).toFixed(decimals).slice(2)
              : "";
            if (step == 1) {
              ch_last = '';
            }
            return ch_first + ch_rest + ch_last;
          },
          isEmpty(value) {
            if (!!value && value instanceof Array) {
              return value.length < 1;
            }
            if (!!value && typeof value === "object") {
              for (var key in value) {
                if (hasOwnProperty.call(value, key)) {
                  return false;
                }
              }
            }
            return !value;
          },
        },
        created() {
          this.initCart();
        },
      });
      app.mount(el);
    });
  }

  function phoneInputMask() {
    const lang = drupalSettings.path.currentLanguage;
    if (lang == "ru") {
      $(".field--name-field-customer-phone input")
      .inputmask("+7 (999) 999-99-99", {
        "clearIncomplete": true,
        "showMaskOnHover": false,
      });
    } else {
      $(".field--name-field-customer-phone input")
      .inputmask("+9 (999) 999-99-99", {
        "clearIncomplete": true,
        "showMaskOnHover": false,
      });
    }
  }

  if (typeof Drupal === "object") {
    Drupal.behaviors.syncart = {
      attach: function (context) {
        let action = $(context).attr("action");
        if (action && action.length > 1) {
          if (action.substring(0, 10) == "/checkout/") {
            phoneInputMask();
            vueCartApp('#app-cart'); // reinit cart
          }
        }
      },
    };
  }

  String.prototype.replaceAll = function (search, replacement) {
    return this.split(search).join(replacement);
  };
})(this.jQuery);
