<?php

namespace Drupal\syncart\Hook;

/**
 * Commerce CheckoutFlow Alter.
 */
class FormCommerceCheckoutFlowMultistepDefaultAlter {

  /**
   * Hook preprocess.
   */
  public static function hook(&$form, $form_state) {
    $form['actions']['next']['#value'] = t('Next');
    $order = \Drupal::routeMatch()->getParameters()->get('commerce_order');
    // Step 1 Order Information.
    if ($form['#step_id'] == 'order_information') {
      $userId = \Drupal::currentUser()->id();
      if ($userId) {
        $storageOrder = \Drupal::entityTypeManager()->getStorage('commerce_order');
        $ids = $storageOrder
          ->getQuery()
          ->condition('uid', $userId)
          ->sort('created', 'DESC')
          ->range(0, 1)
          ->execute();
        if (!empty($ids)) {
          $id = array_shift($ids);
          $profile = $storageOrder->load($id)->getBillingProfile();
          if (!empty($profile)) {
            $form['payment_information']['billing_information']['field_customer_name']['widget'][0]['value']['#default_value'] = $profile->field_customer_name->value;
            $form['payment_information']['billing_information']['field_customer_surname']['widget'][0]['value']['#default_value'] = $profile->field_customer_surname->value;
            $form['payment_information']['billing_information']['field_customer_phone']['widget'][0]['value']['#default_value'] = $profile->field_customer_phone->value;
            $form['payment_information']['billing_information']['field_customer_email']['widget'][0]['value']['#default_value'] = $profile->field_customer_email->value;
          }
        }
      }
      if (!empty($form['shipping_information'])) {
        $form['shipping_information']['#title'] = t('Contact information');
        $form['shipping_information']['shipments'][0]['shipping_method']['#prefix'] = '<div class="checkout-legend--div">' . t('Shipping Information') . '</div>';
        if (!empty($form['shipping_information']['shipping_profile']['select_address'])) {
          $form['shipping_information']['shipping_profile']['select_address']['#access'] = FALSE;
        }
        if (!empty($form['shipping_information']['shipping_profile']['copy_to_address_book'])) {
          unset($form['shipping_information']['shipping_profile']['copy_to_address_book']);
        }
      }
      elseif (!empty($form['payment_information'])) {
        $form['payment_information']['billing_information']['#prefix'] = '<div class="label">' . t('Contact information') . '</div>';
      }
      if (!empty($form['payment_information']['billing_information']['copy_to_address_book'])) {
        unset($form['payment_information']['billing_information']['copy_to_address_book']);
      }
      if (!empty($form['billing_information']['profile']['copy_to_address_book'])) {
        unset($form['billing_information']['profile']['copy_to_address_book']);
      }
      if (!empty($form['sidebar'])) {
        $form['sidebar']['#access'] = FALSE;
      }
    }
    // Step 2 Review.
    if ($form['#step_id'] == 'review') {
      $form['actions']['next']['#prefix'] = $form['actions']['next']['#suffix'];
      unset($form['actions']['next']['#suffix']);
    }
    // Step 3 Payment.
    if ($form['#step_id'] == 'payment') {

    }
    // Step 4 Complete.
    if ($form['#step_id'] == 'complete') {
      $data = [];
      $order_items = $order->order_items;
      if (!empty($order_items)) {
        $config_synapse = \Drupal::config('synapse.settings');
        $data = [
          'transaction_id' => $order->order_id->value,
          'currency' => $order->total_price->currency_code,
          'value' => $order->total_price->number,
          'items' => [],
          'send_to' => $config_synapse->get('ga4-id') ?? '',
        ];
        foreach ($order_items as $id => $item) {
          $order_item = $item->entity;
          $product_info = \Drupal::service('syncart.render')->getVariationInfo($order_item);
          $product = [
            'item_id' => $product_info['vid'],
            'item_name' => $product_info['title'],
            'price' => $product_info['number'],
            'item_category' =>$product_info['catalog'],
            'quantity' => $product_info['quantity'],
          ];
          $data['items'][] = $product;
        }
      }
      $form['#attached']['drupalSettings']['metrika']['dataLayerPurchase'] = $data;
      $form['#attached']['library'][] = 'syncart/ecommerce';
    }
  }

}
