<?php

namespace Drupal\syncart\Hook;

use Drupal\taxonomy\Entity\Term;
use Drupal\paragraphs\Entity\Paragraph;

/**
 * Hook preprocess page class.
 */
class PreprocessCommerceProduct {

  /**
   * Hook.
   */
  public static function hook(&$variables) {
    $variables['cart'] = [
      '#theme' => 'syncart-cart-field',
      '#data' => self::getProductVariation($variables['product_entity'], $variables['elements']['#view_mode']),
    ];

    $variables['favorite'] = [
      '#theme' => 'syncart-favorite-field',
      '#data' => [
        'uid' => \Drupal::currentUser()->id(),
        'product' => $variables['product_entity'],
      ],
    ];
  }

  /**
   * Get product variation price.
   */
  public static function getProductVariation($product, $view_mode) {
    $full = $view_mode == 'full' ? TRUE : FALSE;
    $unit_name = '';
    $qstep = 1;
    if ($product->hasField('field_unit')) {
      $unit = $product->field_unit->entity;
      if (!empty($unit)) {
        $unit_name = $unit->getName();
      }
    }
    if ($product->hasField('field_package')) {
      $qstep = $product->field_package->value;
    }
    $result = FALSE;
    $conf = \Drupal::config('core.entity_view_display.commerce_product_variation.variation.default');
    $variation_definitions = $conf->getRawData()['content'];
    $attribute_original = \Drupal::entityTypeManager()
      ->getStorage('commerce_product_attribute')
      ->getQuery()
      ->execute();
    $attribute_weight = [];
    $weight = 100;
    foreach ($attribute_original as $key => $value) {
      if (array_key_exists("attribute_{$key}", $variation_definitions)) {
        $attribute_weight[$key] = $variation_definitions["attribute_{$key}"]['weight'];
      }
      else {
        if ($key == 'color') {
          $attribute_weight[$key] = 1;
        }
        else {
          $attribute_weight[$key] = $weight++;
        }
      }
    }
    $config = \Drupal::config('syncart.settings');
    $check_store = (bool) $config->get('check_store') ?? FALSE;
    $donation = (bool) $config->get('donation') ?? FALSE;
    $note = (bool) $config->get('note') ?? FALSE;
    $novariation = $config->get('no_variation_text') ?? '';
    $showquantity = $config->get('show_quantity') ?? 0;
    $quantity_instead_add = $config->get('quantity_instead_add') ?? 0;
    $currency_storage = \Drupal::entityTypeManager()->getStorage('commerce_currency');
    $currency_formatter = \Drupal::service('commerce_price.currency_formatter');
    $prices = [];
    $attributes = [];
    foreach ($product->variations as $item) {
      if (empty($item->entity)) {
        continue;
      }
      $variation = $item->entity;
      if (!is_object($variation->field_stock)) {
        $stock = 0;
      }
      else {
        $stock = (int) $variation->field_stock->value;
      }
      if (!is_object($variation) || (!$variation->status->value)) {
        continue;
      }
      $priceObject = $variation->getPrice();
      if (!is_object($priceObject)) {
        continue;
      }
      $stores = [];
      if (is_object($variation->field_stocks)) {
        $paragraphs = $variation->field_stocks->getValue();
        if (!empty($paragraphs)) {
          foreach ($paragraphs as $store) {
            $paragraph = Paragraph::load($store['target_id']);
            if ($paragraph->field_stores_store->entity->status) {
              $stores[] = [
                'name' => $paragraph->field_stores_store->entity->getName(),
                'quantity' => $paragraph->field_stores_stock->value,
              ];
            }
          }
        }
      }
      // dsm($stores);
      $attribute_item = [];
      $weight = 1;
      foreach ($variation->getAttributeValues() as $key => $value) {
        $attribute_label = $value->getAttribute()->label();
        $attribute_original = $value->getAttribute()->getOriginalId();
        $attribute_item[] = [
          'id' => $value->id(),
          'label' => $attribute_label,
          'name' => $value->getName(),
        ];
        if ($check_store && $stock == 0) {
          continue;
        }
        $attribute = [
          'id' => $value->id(),
          'name' => $value->getName(),
          'label' => $attribute_original,
        ];
        if ($value->hasField('field_color') && !empty($value->get('field_color')->getValue()[0])) {
          $attribute['value'] = $value->get('field_color')->getValue()[0];
        }
        if ($value->hasField('field_hex') && !empty($value->get('field_hex')->getValue()[0])) {
          $attribute['value'] = $value->get('field_hex')->getValue()[0];
        }
        if (empty($attributes)) {
          $attributes[$attribute_weight[$attribute_original]][$attribute_label][] = $attribute;
        }
        if (empty($attributes[$attribute_weight[$attribute_original]][$attribute_label])) {
          $attributes[$attribute_weight[$attribute_original]][$attribute_label][] = $attribute;
        }
        elseif (!in_array($attribute, $attributes[$attribute_weight[$attribute_original]][$attribute_label])) {
          $attributes[$attribute_weight[$attribute_original]][$attribute_label][] = $attribute;
        }
      }
      $price = $priceObject->getNumber();
      $currency_code = $variation->getPrice()->getCurrencyCode();
      /** @var \Drupal\commerce_price\Entity\CurrencyInterface $currency */
      $currency = $currency_storage->load($currency_code);
      $old_price = FALSE;
      if ($variation->field_oldprice && $variation->field_oldprice->value) {
        $old_price = $variation->field_oldprice->value;
      }
      if ($product->hasField('field_catalog')) {
        $catalog = $product->field_catalog->entity->getName();
      }
      $info['catalog'] = $catalog ?? '';
      $prices[] = [
        'vid' => $variation->id(),
        'product_id' => $product->id(),
        'status' => $variation->status->value,
        'title' => $variation->getTitle(),
        'price' => $price,
        'old_price' => $old_price,
        'currency_code' => $currency_code,
        'price_format' => !empty($price) ? $currency_formatter->format($price, $currency_code, []) : '',
        'oldprice_format' => !empty($old_price) ? $currency_formatter->format($old_price, $currency_code, []) : '',
        'currency' => $currency->getSymbol(),
        'catalog' => $catalog ?? '',
        'attributes' => $attribute_item,
        'stock' => $stock,
        'stores' => $stores,
      ];
    }
    ksort($attributes);
    $attributes_sorted = [];
    $first = TRUE;
    $master = [];
    foreach ($attributes as $sorted) {
      foreach ($sorted as $key => $value) {
        if ($first) {
          foreach ($value as $id) {
            $master[] = $id['id'];
          }
        }
        $attributes_sorted[$key] = $value;
      }
      $first = FALSE;
    }
    $result = [
      'product' => $product,
      'settings' => [
        'status' => (int) $product->status->value,
        'unit' => $unit_name,
        'qstep' => (float) $qstep,
        'fullstate' => $full,
        'checkstore' => $check_store,
        'showquantity' => $showquantity,
        'quantity_instead_add' => $quantity_instead_add,
        'novariation' => $novariation,
        'masterarray' => $master,
        'donation' => $donation,
        'note' => $note,
      ],
      'prices' => $prices,
      'attributes' => $attributes_sorted,
      'uid' => \Drupal::currentUser()->id(),
    ];
    return $result;
  }

}
